/* opt_usv v0.4 usv2.c
 *
 * Copyright (C) by Frank Gockel 1995-1997
 * Modified      by Andreas Krah 2003
 * Distributed under the Gnu General Public Licence
 *
 */

#include<unistd.h>
#include<stdio.h>
#include<syslog.h>
#include<sys/ioctl.h>
#include<signal.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<stdlib.h>

#define DEFAULT_DEVICE "/dev/usv"
#define VERSION "0.4"

void set_rts(int value,int fd)
{ int a;
  
  ioctl(fd,TIOCMGET,&a);
  a&=~TIOCM_RTS;
  if(value)a|=TIOCM_RTS;
  ioctl(fd,TIOCMSET,&a);
}

void set_dtr(int value,int fd)
{ int a;

  ioctl(fd,TIOCMGET,&a);
  a&=~TIOCM_DTR;
  if(value)a|=TIOCM_DTR;
  ioctl(fd,TIOCMSET,&a);
}

int get_dcd(int fd)
{ int a;

  ioctl(fd,TIOCMGET,&a);
  if(a&TIOCM_CAR)return 1;
  return 0;
}

int get_ri(int fd)
{ int a;

  ioctl(fd,TIOCMGET,&a);
  if(a&TIOCM_RNG)return 1;
  return 0;
}

int get_dsr(int fd)
{ int a;

  ioctl(fd,TIOCMGET,&a);
  if(a&TIOCM_DSR)return 1;
  return 0;
}

int get_cts(int fd)
{ int a;

  ioctl(fd,TIOCMGET,&a);
  if(a&TIOCM_CTS)return 1;
  return 0;
}

void signal_handler(int sig)
{ syslog(LOG_WARNING,"terminated by signal %d\n",sig);
  exit(0); 
}

int main(int argc,char*argv[])
{ int linefail=0;
  int battlow=0;
  int noconnect=0;
  char default_device[]=DEFAULT_DEVICE;
  char* device=default_device;
  int fd;
  int dummy;
  int i;
  
  if(argc>2)
  { fprintf(stderr,"Usage: %s [device]\nDefault device is %s\n",argv[0],
            DEFAULT_DEVICE);
    exit(1);
  }
  
  if(argc==2)device=argv[1];

  /* start system logging */
  openlog(argv[0],LOG_CONS|LOG_PID,LOG_DAEMON);
    
  fd=open(device,O_NDELAY);
  if(fd<0)
  { fprintf(stderr,"%s: unable to open device %s, exiting.\n",argv[0],device);
    syslog(LOG_WARNING,"unable to open device %s, exiting.\n",device);
    exit(2);
  }
  
  if(ioctl(fd,TIOCMGET,&dummy)<0)
  { fprintf(stderr,"%s: unable to get RS232 flags (device not a serial port?), exiting.\n",argv[0]);
    syslog(LOG_WARNING,"unable to get RS232 flags (device not a serial port?), exiting.\n");
    close(fd);
    exit(3);
  }
  
  fprintf(stderr,"%s: USV daemon version %s starting...\n",argv[0],VERSION);
  syslog(LOG_WARNING,"USV daemon version %s starting...\n",VERSION);
  system("export USV_STATUS=\"1\"");
  set_dtr(0,fd);
  set_rts(0,fd);
  sleep(2);
  
  if(get_dsr(fd)!=0&&get_dcd(fd)==0)
  { fprintf(stderr,"%s: usv not connected on startup!\7\n",argv[0]);
    syslog(LOG_WARNING,"usv not connected on startup!\n");
    system("export USV_STATUS=\"2\"");
    dummy=3;
  }
  else dummy=0;
  
  /* go into background */
  i=fork();
  if(i<0)
  { fprintf(stderr,"%s: system error: cannot fork, exiting.\n",argv[0]);
    syslog(LOG_WARNING,"sytem error: cannot fork, exiting.\n");
    exit(4);
  }
  
  /* parent returns with exit code 3 if usv not connected */
  if(i>0)exit(dummy);
  
  /* install signal handlers */
  signal(SIGHUP,signal_handler);
  signal(SIGINT,signal_handler);
  signal(SIGQUIT,signal_handler);
  signal(SIGILL,signal_handler);
  signal(SIGIO,signal_handler);
  signal(SIGTERM,signal_handler);
  signal(SIGSEGV,signal_handler);
  
  syslog(LOG_WARNING,"startup successful\n");
  system("date +\"%d. %B %Y  %T : OPT-USV v0.4 startup\" >> /etc/usv/usv.log");

  for(;;)
  {  sleep(10);
     
     if(get_dsr(fd)!=0&&get_dcd(fd)==0)
     { if(noconnect==0)
       { syslog(LOG_WARNING,"usv not connected, waiting for connection.\n");
         system("export USV_STATUS=\"2\"");
         system("/etc/usv/noconnect");
       }
       noconnect=1;
       continue;
     }
     else 
     { if(noconnect!=0)syslog(LOG_WARNING,"usv connection ok, continuing.\n");
       noconnect=0;
     }

     if(get_dcd(fd)!=0&&linefail==0)
     { linefail=1;
       syslog(LOG_WARNING,"line failure\n");
       system("export USV_STATUS=\"3\"");
       system("/etc/usv/linefail");
       sleep(4);
       set_dtr(1,fd);
     }
     
     if(get_dcd(fd)==0&&linefail!=0)
     { linefail=0;battlow=0;
       syslog(LOG_WARNING,"line is back\n");
       system("export USV_STATUS=\"1\"");
       system("/etc/usv/lineback");
       set_dtr(0,fd);
     }
     
     if(get_cts(fd)!=0&&battlow==0)
     { battlow=1;
       syslog(LOG_WARNING,"battery low\n");
       system("export USV_STATUS=\"4\"");
       system("/etc/usv/battlow");
     }
  }
   
}
